////////////////////////////////////////////////////////////////////////////////
//
//   LLVMTA - Timing Analyser performing worst-case execution time analysis
//     using the LLVM backend intermediate representation
//
// Copyright (C) 2013-2015  Saarland University
//
// This file is distributed under the Saarland University Software Release
// License. See LICENSE.TXT for details.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESSED OR IMPLIED
// WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE SAARLAND UNIVERSITY, THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED OR OTHER LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, TORT OR OTHERWISE, ARISING IN ANY WAY FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH A DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef RESULTDUMPPASS_H
#define RESULTDUMPPASS_H

#include "ARM.h"
#include "ARMMachineFunctionInfo.h"
#include "llvm/CodeGen/MachineFunctionPass.h"
#include "llvm/Support/CommandLine.h"
#include "llvm/Target/TargetMachine.h"
#include "llvm/CodeGen/TargetInstrInfo.h"
#include <iostream>
#include "StaticAddressProvider.h"

using namespace llvm;

namespace TimingAnalysisPass {
	
	/**
	 * Pass that prints the resulting assembler for the given program if option enable-asm-dump is true.
	 * In any case, it checks that the program with its instructions adheres to our
	 * implicit assumptions and gives reasonable error messages to the user.
	 */
  class AsmDumpAndCheckPass : public MachineFunctionPass {
  	public:
    static char ID;
    AsmDumpAndCheckPass(TargetMachine &TM);

		bool runOnMachineBasicBlock(MachineBasicBlock &MBB);
    bool runOnMachineFunction(MachineFunction &F);
		bool doFinalization(Module&);

    virtual llvm::StringRef getPassName() const {
      return "ARM Timing Analysis Result Dump Pass";
    }

  	private:
		template <llvm::Triple::ArchType ARCH>
		void checkInstruction(const MachineInstr& I);

  	TargetMachine &TM;
		// Keep track on whether we saw an unknown instruction
		bool seenUnknownInstruction;
  };

}

namespace llvm {
	FunctionPass* createAsmDumpAndCheckPass(TargetMachine &TM);
}

#endif
