////////////////////////////////////////////////////////////////////////////////
//
//   LLVMTA - Timing Analyser performing worst-case execution time analysis
//     using the LLVM backend intermediate representation
//
// Copyright (C) 2013-2015  Saarland University
//
// This file is distributed under the Saarland University Software Release
// License. See LICENSE.TXT for details.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESSED OR IMPLIED
// WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE SAARLAND UNIVERSITY, THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED OR OTHER LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, TORT OR OTHERWISE, ARISING IN ANY WAY FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH A DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef PROGANA_LATTICES_H_
#define PROGANA_LATTICES_H_

#include <boost/concept_check.hpp>
#include <boost/concept/requires.hpp>
#include "Memory/progana/Lattice.h"

namespace TimingAnalysisPass {

namespace concepts {

template <class X>
struct TotalOrder : boost::LessThanComparable<X> {};

template <class X>
struct PartialOrder
{
	BOOST_CONCEPT_USAGE(PartialOrder)
	{
		po_lt(x, y);
		po_leq(x, y);
		po_gt(x, y);
		po_geq(x, y);
	}

private:
	const X x, y;
};

template <class X>
struct JoinOrMeetSemiLattice
	: PartialOrder<X>
	, boost::EqualityComparable<X>
{};

template <class X>
struct JoinSemiLattice : JoinOrMeetSemiLattice<X>
{
	BOOST_CONCEPT_USAGE(JoinSemiLattice)
	{
		x.join(cX);
		x = cX | cX;
		x |= cX;
	}

private:
	X x;
	const X cX;
};

template <class X>
struct MeetSemiLattice : JoinOrMeetSemiLattice<X>
{
	BOOST_CONCEPT_USAGE(MeetSemiLattice)
	{
		x.meet(cX);
		x = cX & cX;
		x &= cX;
	}

private:
	X x;
	const X cX;
};

template <class X>
struct CompleteLattice : JoinSemiLattice<X>	, MeetSemiLattice<X> {};

}  // namespace concepts

} // namespace TimingAnalysisPass

#endif /*CONCEPTS_LATTICES_H_*/
