#ifndef CACHE_UTILS_H
#define CACHE_UTILS_H

#include "Util/AbstractAddress.h"
#include "LLVMPasses/StaticAddressProvider.h"
#include "Util/Options.h"
#include "Util/Util.h"
#include "Memory/CacheTraits.h"

namespace TimingAnalysisPass {
template<dom::cache::CacheTraits* CacheConfig>
typename dom::cache::CacheTraits::AddressType getCachelineAddress(typename dom::cache::CacheTraits::AddressType addr)
{
	return addr & ~(CacheConfig->LINE_SIZE-1);
}

template<dom::cache::CacheTraits* CacheConfig>
typename dom::cache::CacheTraits::TagType getTag(typename dom::cache::CacheTraits::AddressType addr)
{
	return (addr / CacheConfig->LINE_SIZE) / CacheConfig->N_SETS;
}

template<dom::cache::CacheTraits* CacheConfig>
typename dom::cache::CacheTraits::TagType getTag(AbstractAddress addr)
{
	AddressInterval itv = addr.getAsInterval();
	assert(getCachelineAddress<CacheConfig>(itv.lower()) == getCachelineAddress<CacheConfig>(itv.upper()));

	return getTag<CacheConfig>(itv.lower());
}

/* returns (in cycles) how long the transfer of a cache line between cache and
 * main memory takes */
unsigned getCachelineMemoryLatency(CacheType type);

template<dom::cache::CacheTraits* CacheConfig>
unsigned getPerSetSize(const GlobalVariable *ds)
{
	unsigned size = staticAddressProvider->getArraySize(ds);
	unsigned base = staticAddressProvider->getGlobalVarAddress(ds);

	/* If the array is not aligned at cacheline boundaries we have to
	 * account for the initial part of the cacheline */
	size += base - getCachelineAddress<CacheConfig>(base);

	unsigned cacheSizePerAssoc = CacheConfig->N_SETS * CacheConfig->LINE_SIZE;

	unsigned perSetSize = size / cacheSizePerAssoc;

	if (size % cacheSizePerAssoc != 0) {
		++perSetSize;
	}

	return perSetSize;
}

}

#endif
