////////////////////////////////////////////////////////////////////////////////
//
//   LLVMTA - Timing Analyser performing worst-case execution time analysis
//     using the LLVM backend intermediate representation
//
// Copyright (C) 2013-2015  Saarland University
// Copyright (C) 2014-2015  Claus Faymonville
//
// This file is distributed under the Saarland University Software Release
// License. See LICENSE.TXT for details.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESSED OR IMPLIED
// WARRANTIES, INCLUDING BUT NOT LIMITED TO, THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE SAARLAND UNIVERSITY, THE
// CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING BUT NOT LIMITED TO PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED OR OTHER LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, TORT OR OTHERWISE, ARISING IN ANY WAY FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH A DAMAGE.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef STATISTICS_H
#define STATISTICS_H

#include <map>
#include <fstream>

namespace TimingAnalysisPass {

class Statistics {

public:

	static Statistics& getInstance();

	void startMeasurement(std::string identifier);

	void stopMeasurement(std::string identifier);

	void dump(std::ostream& mystream);

private:

	Statistics(){};

	// declaring, but not implementing copy constructor and assign operator
	Statistics(const Statistics& stats) = delete;
	void operator=(const Statistics& stats) = delete;

	void checkMemoryUsage(std::string identifier);

	unsigned getCurrentMemoryUsage();

	void startTimer(struct timeval* timer);

	void stopTimer(struct timeval* timer);

	// 0: memusage, 1: time usage, 2: final (we saw a stop)
	typedef std::tuple<long long, struct timeval, bool> Measurement;

	std::map<std::string, Measurement> measurements;

	long long memoryUsage = 0;

	struct timeval startTime, endTime;

};

}

#endif
