#include "LLVMPasses/StaticAddressProvider.h"
#include "Util/AbstractAddress.h"
#include "Util/Options.h"

namespace TimingAnalysisPass {
/* constructors */
AbstractAddress::AbstractAddress(Address addr)
{
	this->value.interval = AddressInterval(addr,addr);
	this->type = AddressType::INTERVAL;
}

AbstractAddress::AbstractAddress(AddressInterval itv)
{
	this->value.interval = itv;
	this->type = AddressType::INTERVAL;
}

AbstractAddress::AbstractAddress(const GlobalVariable *gv)
{
	assert(gv);
	if(!ArrayAnalysis) {
		*this = AbstractAddress::getUnknownAddress();
		return;
	}
	this->value.array = gv;
	this->type = AddressType::ARRAY;
}

AbstractAddress AbstractAddress::getUnknownAddress()
{
	AbstractAddress addr (AddressInterval(0,0xffffffff));
	return addr;
}

/* accessors */
bool AbstractAddress::isArray() const
{
	return type==AddressType::ARRAY;
}
bool AbstractAddress::isPrecise() const
{
	return this->type == AddressType::INTERVAL &&
	       value.interval.lower() == value.interval.upper();
}

const GlobalVariable* AbstractAddress::getArray() const
{
	if (!this->isArray()) {
		return nullptr;
	}
	return value.array;
}

AddressInterval AbstractAddress::getAsInterval() const
{
	switch(this->type) {
		case AddressType::INTERVAL:
			return value.interval;
		case AddressType::ARRAY: {
			unsigned base = staticAddressProvider->getGlobalVarAddress(value.array);
			unsigned size = staticAddressProvider->getArraySize(value.array);
			return AddressInterval(base,base+size-4);
		}
		default:
			assert(0 && "UNREACHABLE");
	}
}

/* Comparisons */
bool AbstractAddress::isSameInterval(const AbstractAddress other) const
{
	AddressInterval this_itv = this->getAsInterval();
	AddressInterval other_itv = other.getAsInterval();
	return this_itv.lower() == other_itv.lower() && this_itv.upper() == other_itv.upper();
}

}
